;    This program is free software: you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation, either version 3 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program.  If not, see <http://www.gnu.org/licenses/>.

Scriptname LightingBall extends ObjectReference  

ObjectReference Property thisPlayer Auto Hidden
ObjectReference Property light_1 Auto
ObjectReference Property light_2 Auto
ObjectReference Property light_3 Auto
ObjectReference Property light_4 Auto
Bool  Property allLightsOn Auto Hidden
Bool  Property allLightsOff Auto Hidden
Bool  Property OnTimer Auto
Bool  Property specialCaseNoDisables Auto
Bool  Property specialCaseNoEnables Auto
Bool  Property debugModeOn = True Auto
Float Property OnTimerAutoUpdate Auto
Float Property OnTimerBeginTime Auto
Float Property OnTimerEndTime Auto


EVENT onInit()
	
	thisPlayer = Game.getPlayer()

ENDEVENT

EVENT onCellAttach()
	
	IF ((OnTimerAutoUpdate > 0) && (OnTimer))	
		IF (debugModeOn)
			IF (specialCaseNoDisables || specialCaseNoEnables)
				Debug.MessageBox("debugModeOn: " + Self + " is on a timer and auto-updating every " + OnTimerAutoUpdate + " seconds, however its specialCaseNoDisables and/or specialCaseNoEnables flag is set.  Disabling. ")
				disable()	
				return
			ELSEIF (!(mask(1, 0, 0, 0) || mask(1, 1, 0, 0) || mask(1, 1, 1, 0) || mask(1, 1, 1, 1)))
				Debug.MessageBox("DebugModeOn: " + Self + " has mismatched lights!  I.E. light_1 must be set before you can set light_2.  Disabling. ")
				disable()
				return
			ENDIF
		ENDIF
		
		allLightsOn = (light_1.isEnabled() && ((light_2 == None) || (light_2.isEnabled())) && ((light_3 == None) || (light_3.isEnabled())) && ((light_4 == None) || (light_4.isEnabled())))
		allLightsOff =	(light_1.isDisabled() && ((light_2 == None) || (light_2.isDisabled())) && ((light_3 == None) || (light_3.isDisabled())) && ((light_4 == None) || (light_4.isDisabled())))
		
		IF (isEnabled())
			onUpdate()
			registerForUpdate(OnTimerAutoUpdate)	
		ENDIF
				
	ENDIF
	
ENDEVENT


EVENT onCellDetach()

	IF ((OnTimerAutoUpdate > 0) && (OnTimer))
		unregisterForUpdate()
	ENDIF
	
ENDEVENT


EVENT OnUpdate()	
	
	IF (isBetweenTimes(OnTimerBeginTime, OnTimerEndTime))
		IF (!allLightsOn || (OnTimerBeginTime == OnTimerEndTime))							
			light_1.EnableNoWait()
			IF (light_2 != None)
				light_2.EnableNoWait()
				IF (light_3 != None)
					light_3.EnableNoWait()
					IF (light_4 != None)
						light_4.EnableNoWait()
					ENDIF
				ENDIF
			ENDIF		
			allLightsOn = True	
			allLightsOff = False
		ENDIF
	ELSEIF (!allLightsOff)		
		light_1.DisableNoWait()
		IF (light_2 != None)
			light_2.DisableNoWait()
			IF (light_3 != None)
				light_3.DisableNoWait()
				IF (light_4 != None)
					light_4.DisableNoWait()
				ENDIF
			ENDIF
		ENDIF
		allLightsOff = True
		allLightsOn  = False
	ENDIF	

ENDEVENT

Float FUNCTION getDistanceToPlayer()

	return(getDistance(thisPlayer))
		
ENDFUNCTION


Bool FUNCTION enableLights()
	
	IF ((!specialCaseNoEnables) && ((!OnTimer) || (isBetweenTimes(OnTimerBeginTime, OnTimerEndTime))))
		light_1.EnableNoWait()
		IF (light_2 != None)
			light_2.EnableNoWait()
			IF (light_3 != None)
				light_3.EnableNoWait()
				IF (light_4 != None)
					light_4.EnableNoWait()
				ENDIF
			ENDIF
		ENDIF	
		
		return True
		
	ENDIF
	
	return False

ENDFUNCTION


FUNCTION disableLights()

	IF (!specialCaseNoDisables)
		light_1.DisableNoWait()
		IF (light_2 != None)
			light_2.DisableNoWait()
			IF (light_3 != None)
				light_3.DisableNoWait()
				IF (light_4 != None)
					light_4.DisableNoWait()
				ENDIF
			ENDIF
		ENDIF
	ENDIF

ENDFUNCTION


FUNCTION disableNoWait(bool notUsed = false)
	
	disableLights()	
	allLightsOff = True
	
	IF ((OnTimerAutoUpdate > 0) && (OnTimer))
		unregisterForUpdate()
	ENDIF
	
	Parent.disableNoWait()
	
	
ENDFUNCTION


FUNCTION enableNoWait(bool notUsed = false)
	
	Parent.enable()
	allLightsOn = enableLights()		
	
	IF (!allLightsOn)
		disableLights()
		allLightsOff = True
	ENDIF
	
	IF ((OnTimerAutoUpdate > 0) && (OnTimer))
		registerForUpdate(OnTimerAutoUpdate)
	ENDIF
	
ENDFUNCTION


Bool FUNCTION isBetweenTimes(float time1, float time2) global
	
	IF (time1 != time2)
		float currentTime = Utility.GetCurrentGameTime()
		currentTime -= (currentTime as int)
		currentTime *= 24
		
		IF (time1 > time2)
			return ((currentTime >= time1) || (currentTime < time2))
		ENDIF
	
		return ((currentTime >= time1) && (currentTime < time2))
	ENDIF
	
	return True
	
ENDFUNCTION 


Bool FUNCTION hasSameLights (LightingBall other)

	IF     (light_1 == None) 
		 return False
	ELSEIF (light_1 == other.light_1) 
		return True
	ELSEIF (light_1 == other.light_2) 
		return True
	ELSEIF (light_1 == other.light_3) 
		return True
	ELSEIF (light_1 == other.light_4) 
		return True
		
	ELSEIF (light_2 == None) 
		return False
	ELSEIF (light_2 == other.light_1) 
		return True
	ELSEIF (light_2 == other.light_2) 
		return True
	ELSEIF (light_2 == other.light_3) 
		return True
	ELSEIF (light_2 == other.light_4) 
		return True
		
	ELSEIF (light_3 == None) 
		return False
	ELSEIF (light_3 == other.light_1) 
		return True
	ELSEIF (light_3 == other.light_2) 
		return True
	ELSEIF (light_3 == other.light_3) 
		return True
	ELSEIF (light_3 == other.light_4) 
		return True
		
	ELSEIF (light_4 == None) 
		return False
	ELSEIF (light_4 == other.light_1) 
		return True
	ELSEIF (light_4 == other.light_2) 
		return True
	ELSEIF (light_4 == other.light_3) 
		return True
	ELSEIF (light_4 == other.light_4) 
		return True
	ENDIF
	
	return False
	
ENDFUNCTION


Bool FUNCTION mask (Bool b1, Bool b2, Bool b3, Bool b4)

	return ((b1 == (light_1 != None)) && (b2 == (light_2 != None)) && (b3 == (light_3 != None)) && (b4 == (light_4 != None)))
	
ENDFUNCTION


Bool FUNCTION doSelfTest()

	IF (specialCaseNoDisables && specialCaseNoEnables)
		return (mask(0, 0, 0, 0) || mask(1, 0, 0, 0) || mask(1, 1, 0, 0) || mask(1, 1, 1, 0) || mask(1, 1, 1, 1))		
	ENDIF
	
	return (mask(1, 0, 0, 0) || mask(1, 1, 0, 0) || mask(1, 1, 1, 0) || mask(1, 1, 1, 1))		
		
ENDFUNCTION
